package com.k2.tornado.midlet;

import javax.microedition.lcdui.*;
import javax.microedition.lcdui.game.*;
import javax.microedition.media.*;
import javax.microedition.media.control.*;

import com.k2.tornado.object.ControlableSprite;
import com.k2.tornado.layers.*;

/**
 * <p>Title: Tornado</p>
 * <p>Description: A cat's adventure in a land of weirdos</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: K-Square</p>
 * @author Steffen Kluge
 * @version 0.1
 * <p>
 * GameScreen is the Displayable that shows all game action.
 * It is implemented as a thread whose run-method represents
 * the main game loop. All key state changes are monitored
 * there.
 *
 */
public class GameScreen extends GameCanvas implements Runnable
{
	/**
	 * screen width
	 */
	private int m_width;
	/**
	 * screen height
	 */
	private int m_height;
	/**
	 * flag that indicates if the game loop is to keep rolling
	 */
	private boolean m_bFinished = false;
	/**
	 * the thread object that'll run the game loop
	 */
	private Thread m_gameLoop;
	/**
	 * the core modul who must be notified when the back-key is pressed
	 * command listening isn't too useful in threaded fullscreen displayables
	 */
	private TornadoCore m_core = null;
	/**
	 * the midi player
	 */
	private Player m_midi = null;
	/**
	 * The LayerManager object will store references to all visual data.
	 * Thus we'll need only one paint-call to draw all layers we have.
	 */
	private LayerManager m_layerManager = new LayerManager();
	/**
	 * the object that takes key input
	 */
	private ControlableSprite m_avatar = null;

	/** Test Stage built from Tile Studio*/
	private BaseLayer m_stage;

	/**
	 * Constructs the GameScreen object
	 * @param core reference to the calling TornadoCore object whose commandAction will be invoked
	 */
	public GameScreen(TornadoCore core)
	{
		// do not supress key events
		super(true);

		m_core = core;
	}

	public void initScreen() throws Exception
	{
		// reset the finished-flag
		m_bFinished = false;

		// activate Fullscreen mode
		setFullScreenMode(true);

		// create the thread
		m_gameLoop = new Thread(this);

		// Create the audio player
		// m_midi = Manager.createPlayer(this.getClass().getResourceAsStream("/com/k2/tornado/media/music/midi/bennyjet.mid"), "audio/midi");

		// create stage
		m_stage = new TestStageLayer();

		// create the avatar
		if (m_avatar == null) {
			Image img = Image.createImage("/com/k2/tornado/media/images/layers/avatar_20x20.png");
			// set the display dimensions to 0x0 as we cannot be sure right now that they're correct... run() will pass the right dimension later!
			m_avatar = new ControlableSprite(img, 20, 20, 0, 0, m_stage);
			// position the avatar
			m_stage.setAvatarPosition(m_avatar);
			// set the reference pixel position
			m_avatar.defineReferencePixel(m_avatar.getWidth() >> 1, m_avatar.getHeight());
			m_avatar.setFrame(0);
			// append the avatar to the layer manager
		}
		else {
			m_layerManager.remove(m_avatar);
		} //else

		// append the layers to the manager
		m_layerManager.append(m_avatar);
		// m_layerManager.append(m_stage.getTiledLayer());

	}

	public void startLoop()
	{
		try {
			// initialize screen
			initScreen();
			// start the thread
			m_gameLoop.start();
			// start music
			if (m_midi != null) {
				m_midi.start();

			}
		}
		catch (Exception e) {
			e.printStackTrace();
		} //catch
	}

	public void stopLoop()
	{
		try {
			// indicate that the thread shall stop
			m_bFinished = true;
			if (m_midi != null) {
				m_midi.stop();
				m_midi.deallocate();
			} //if
			// perform clean up if neccessary
		}
		catch (MediaException ex) {
			ex.printStackTrace();
		}
	}

	/** Required paint implementation */
	public void paint(Graphics g)
	{
		/** @todo Add paint codes */
	}

	public void run()
	{
		try {
			m_gameLoop.sleep(500);
			// get the screen dimensions
			m_width = getWidth();
			m_height = getHeight();
			m_layerManager.setViewWindow(0, 0, m_width, m_height);
			// pass the display dimension to the differnt layers
			m_avatar.setDisplaySize(m_width, m_height);
			m_stage.setDisplaySize(m_width, m_height);

			// get the graphics object
			Graphics g = getGraphics(); // this method delivers the offscreen plain
			// g.clipRect(0, 16, m_width, m_height-32);

			// we must monitor the time it takes to paint a screen
			long t1, duration = 0;

			while (!m_bFinished) {
				// get the milliseconds
				t1 = System.currentTimeMillis();

				// handle the key states
				m_avatar.handleKeyStates(getKeyStates());

				// clear the offscreen plain
				g.setColor(0xE0E8FF);
				g.fillRect(0, 0, m_width, m_height);

				// animate the landscape
				m_stage.animate(); // this replaces sequence indices with static tiles

				// draw the layers
				// m_layerManager.paint(g, 0, 0);
				m_stage.paint(g);
				m_avatar.paint(g);

				g.setColor(0x000000);
				String s = String.valueOf(duration);
				g.drawString("last Frame: " + s + " ms.", 0, 20, 0);

				// show what we've drawn
				flushGraphics();

				// lets measure
				duration = System.currentTimeMillis() - t1;

				m_gameLoop.sleep(25);
			} //while
		}
		catch (Exception e) {
			e.printStackTrace();
		} //catch
	}

	protected void keyPressed(int keyCode)
	{
		if (keyCode == -7) {
			// stop the loop here
			if (m_core != null) {
				m_core.commandAction(m_core.backCommand, this);
			} //if
		}
		super.keyPressed(keyCode);
	}
}
