package com.k2.tornado.object;

import javax.microedition.lcdui.*;
import javax.microedition.lcdui.game.*;

import com.k2.tornado.layers.*;

/**
 * <p>Title: Tornado</p>
 * <p>Description: A cat's adventure in a land of weirdos</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: K-Square</p>
 * @author Steffen Kluge
 * @version 0.1
 */

public class ControlableSprite extends Sprite
{
	/**
	 * the sprite's state
	 */
	private int m_state = SpriteStates.SPRITESTATE_FALLING;
	private int lastState = m_state;

	/**
	 * the tile grid that this sprite can collide with
	 */
	private BaseLayer m_landscape;
	/**
	 * the display width
	 */
	private int m_displayWidth = 0;
	/**
	 * the display height
	 */
	private int m_displayHeight = 0;
	/**
	 * if the sprite gets this close to the display border,
	 * the landscape will be scrolled
	 */
	private int m_inset = 80;

	/**
	 * the velocity when moving left or right
	 */
	private int[] m_hVelocity = {
		0, 1, 2, 4, 6,};
	/**
	 * the index to the hVelocity table
	 */
	private int m_hVelocityIndex = 0;
	/**
	 * the maximum left/right velocity - used when leaping or falling
	 */
	private int m_hVelocityMax = 6;
	private int[] m_vVelocity = {
		 -12, -12, -12, -10, -5, -2};
	private int m_vVelocityIndex = 0;

	/**
	 * Constructs a ControlableSprite object. Controlable sprites are those who
	 * can be moved around with the keys. Thus an application should always
	 * contain only one of these - the avatar object.
	 * @param frames an Image object that contains this sprite's frames
	 * @param frameWidth the width of a frame (all frames have equal dimensions!)
	 * @param frameHeight the height of a frame
	 */
	public ControlableSprite(Image frames, int frameWidth, int frameHeight, int displayWidth, int displayHeight, BaseLayer landscape)
	{
		super(frames, frameWidth, frameHeight);
		m_landscape = landscape;
		setDisplaySize(displayWidth, displayHeight);
	}

	public void handleKeyStates(int keyStates)
	{
		// RIGHT
		if ( (keyStates & GameCanvas.RIGHT_PRESSED) != 0) {
			// we're moving right
			m_state |= SpriteStates.SPRITESTATE_MOVING_RIGHT;
			// thus we can't move or walk left
			m_state &= ~SpriteStates.SPRITESTATE_MOVING_LEFT;
			// do we neither leap or fall?
			if ( (m_state & (SpriteStates.SPRITESTATE_LEAPING | SpriteStates.SPRITESTATE_FALLING)) == 0) {
				// yer, then we're moving
				m_state |= SpriteStates.SPRITESTATE_WALKING;
			}
		}
		else {
			// we're not moving right anymore
			m_state &= ~SpriteStates.SPRITESTATE_MOVING_RIGHT;
		} //else

		// LEFT
		if ( (keyStates & GameCanvas.LEFT_PRESSED) != 0) {
			// we're moving left
			m_state |= SpriteStates.SPRITESTATE_MOVING_LEFT;
			// thus we can't move or walk left
			m_state &= ~SpriteStates.SPRITESTATE_MOVING_RIGHT;
			// do we neither leap or fall?
			if ( (m_state & (SpriteStates.SPRITESTATE_LEAPING | SpriteStates.SPRITESTATE_FALLING)) == 0) {
				// yer, then we're moving
				m_state |= SpriteStates.SPRITESTATE_WALKING;
			}
		}
		else {
			// we're not moving left anymore
			m_state &= ~SpriteStates.SPRITESTATE_MOVING_LEFT;
		} //else

		// UP
		if ( (keyStates & (GameCanvas.DOWN_PRESSED|GameCanvas.UP_PRESSED)) != 0) {
			// do we currently fall?
			if ( (m_state & SpriteStates.SPRITESTATE_FALLING) == 0) {
				// no - so we may leap now
				m_state |= SpriteStates.SPRITESTATE_LEAPING;
				// in case we were walking we must delete that flag
				m_state &= ~SpriteStates.SPRITESTATE_WALKING;
			}
		}
		else {
			// we're not leaping, did we before?
			if ((m_state & SpriteStates.SPRITESTATE_LEAPING) != 0)
			{
				// yer, so we're falling now
				m_state |= SpriteStates.SPRITESTATE_FALLING;
				m_state &= ~SpriteStates.SPRITESTATE_LEAPING;
				// m_vVelocityIndex = 0;
			}//if
		}//else

		// handle the new state
		handleState();
	}

	public void handleState()
	{
		// **************************** WALKING *********************************
		if ( (m_state & SpriteStates.SPRITESTATE_WALKING) != 0) {
			// yer, left or right?
			if ( (m_state & SpriteStates.SPRITESTATE_MOVING_RIGHT) != 0) {
				// walk right
				move(m_hVelocity[m_hVelocityIndex], 0);
				if (m_landscape.checkCollission(this, SpriteStates.SPRITESTATE_MOVING_RIGHT))
				{
					// we can't move on
					m_state &= ~SpriteStates.SPRITESTATE_MOVING_RIGHT;
					m_hVelocityIndex = 0;
				}//if

				// set the transform
				setTransform(Sprite.TRANS_NONE);
			}
			else if ( (m_state & SpriteStates.SPRITESTATE_MOVING_LEFT) != 0) {
				// walk left
				move( -m_hVelocity[m_hVelocityIndex], 0);
				if (m_landscape.checkCollission(this, SpriteStates.SPRITESTATE_MOVING_LEFT))
				{
					// we can't move on
					m_state &= ~SpriteStates.SPRITESTATE_MOVING_RIGHT;
					m_hVelocityIndex = 0;
				}
				// set the transform
				setTransform(Sprite.TRANS_MIRROR);
			}
			else {
				// walk neither left nor right - we can't be a walking anymore
				m_state &= ~SpriteStates.SPRITESTATE_WALKING;
				// reset the horizontal velocity index
				m_hVelocityIndex = 0;
			} //else
			// increase the velocity index
			if (m_hVelocityIndex < m_hVelocity.length - 1) {
				m_hVelocityIndex++;
			}

			// check if we're still on the ground
			if (!m_landscape.checkCollission(this, SpriteStates.SPRITESTATE_WALKING))
			{
				// nope...
				m_state &= ~SpriteStates.SPRITESTATE_WALKING;
				m_state |= SpriteStates.SPRITESTATE_FALLING;
				m_vVelocityIndex = (m_vVelocity.length >> 1) - 1;
			}//if
		}
		// **************************** FALLING  ********************************
		if ( (m_state & SpriteStates.SPRITESTATE_FALLING) != 0) {
			// we're falling - left or right?
			if ( (m_state & SpriteStates.SPRITESTATE_MOVING_RIGHT) != 0) {
				// fall right
				move(m_hVelocityMax, -m_vVelocity[m_vVelocityIndex]);
				if (m_landscape.checkCollission(this, SpriteStates.SPRITESTATE_FALLING))
				{
					// stop falling - position has already been adjusted
					m_state &= ~SpriteStates.SPRITESTATE_FALLING;
					m_vVelocityIndex = 0;
				}//if
				if (m_landscape.checkCollission(this, SpriteStates.SPRITESTATE_MOVING_RIGHT))
				{
					// we can't keep moving right
					m_state &= ~SpriteStates.SPRITESTATE_MOVING_RIGHT;
				}else
				{
					m_hVelocityIndex = m_hVelocity.length-1;
				}//else

				// set the transform
				setTransform(Sprite.TRANS_NONE);
			}
			else if ( (m_state & SpriteStates.SPRITESTATE_MOVING_LEFT) != 0) {
				// left left
				move( -m_hVelocityMax, -m_vVelocity[m_vVelocityIndex]);
				if (m_landscape.checkCollission(this, SpriteStates.SPRITESTATE_FALLING))
				{
					// stop falling - position has already been adjusted
					m_state &= ~SpriteStates.SPRITESTATE_FALLING;
					m_vVelocityIndex = 0;
				}//if
				if (m_landscape.checkCollission(this, SpriteStates.SPRITESTATE_MOVING_LEFT))
				{
					// we can't keep moving left
					m_state &= ~SpriteStates.SPRITESTATE_MOVING_LEFT;
				}else
				{
					m_hVelocityIndex = m_hVelocity.length-1;
				}//else

				// set the transform
				setTransform(Sprite.TRANS_MIRROR);
			}
			else {
				// we're falling straight down without moving left or right
				move(0, -m_vVelocity[m_vVelocityIndex]);
				if (m_landscape.checkCollission(this, SpriteStates.SPRITESTATE_FALLING))
				{
					// stop falling - position has already been adjusted
					m_state &= ~SpriteStates.SPRITESTATE_FALLING;
					m_vVelocityIndex = 0;
				}//if
			} //else

			// decrease the index
			if (m_vVelocityIndex > 0)
				m_vVelocityIndex--;
		}
		// **************************** LEAPING *********************************
		if ( (m_state & SpriteStates.SPRITESTATE_LEAPING) != 0) {
			// we're leaping - left or right?
			if ( (m_state & SpriteStates.SPRITESTATE_MOVING_RIGHT) != 0) {
				// leap right
				move(m_hVelocityMax, m_vVelocity[m_vVelocityIndex]);
				if (m_landscape.checkCollission(this, SpriteStates.SPRITESTATE_LEAPING))
				{
					// we can't leap higher - we must fall now
					m_state |= SpriteStates.SPRITESTATE_FALLING;
					m_state &= ~SpriteStates.SPRITESTATE_LEAPING;
				}else
				{
					// check horizontal movement also
					if (m_landscape.checkCollission(this, SpriteStates.SPRITESTATE_MOVING_RIGHT)) {
						// we can't move right anymore
						m_state &= ~SpriteStates.SPRITESTATE_MOVING_RIGHT;
					} //if
				}//else

				// set the transform
				setTransform(Sprite.TRANS_NONE);
			}
			else if ( (m_state & SpriteStates.SPRITESTATE_MOVING_LEFT) != 0) {
				// leap left
				move( -m_hVelocityMax, m_vVelocity[m_vVelocityIndex]);
				if (m_landscape.checkCollission(this, SpriteStates.SPRITESTATE_LEAPING))
				{
					// we can't leap higher - we must fall now
					m_state |= SpriteStates.SPRITESTATE_FALLING;
					m_state &= ~SpriteStates.SPRITESTATE_LEAPING;
				}else
				{
					if (m_landscape.checkCollission(this, SpriteStates.SPRITESTATE_MOVING_LEFT)) {
						// we can't move right anymore
						m_state &= ~SpriteStates.SPRITESTATE_MOVING_LEFT;
					} //if
				}//else

				// set the transform
				setTransform(Sprite.TRANS_MIRROR);
			}
			else {
				// we're leaping straight up without moving left or right
				move(0, m_vVelocity[m_vVelocityIndex]);
				if (m_landscape.checkCollission(this, SpriteStates.SPRITESTATE_LEAPING))
				{
					// we can't leap higher - we must fall now
					m_state |= SpriteStates.SPRITESTATE_FALLING;
					m_state &= ~SpriteStates.SPRITESTATE_LEAPING;
				}//else
			} //else
			// increase the index
			if (m_vVelocityIndex < m_vVelocity.length - 1) {
				m_vVelocityIndex++;
			}
			else {
				// we can't leap higher - we must fall now
				m_state |= SpriteStates.SPRITESTATE_FALLING;
				m_state &= ~SpriteStates.SPRITESTATE_LEAPING;
				// m_vVelocityIndex = 0;
			} //else
		} //if

		// ************************* SCROLLING **********************************
		int x = getX();
		int y = getY();
		int dx = 0;
		int dy = 0;

		if (x < m_inset)
		{
			// we must scroll left
			dx = m_inset - x;
		}else if (x > m_displayWidth - m_inset)
		{
			// we must scroll right
			dx = (m_displayWidth-m_inset) - x;
		}//else if

		if (y < m_inset)
		{
			// we must scroll left
			dy = m_inset - y;
		}else if (y > m_displayHeight - m_inset)
		{
			// we must scroll right
			dy = (m_displayHeight-m_inset) - y;
		}//else if

		// scroll and adjust the sprite
		if (m_landscape.hMove(dx))
			move(dx, 0);
		if (m_landscape.vMove(dy))
			move(0, dy);

		/*
		// report the state
		if (m_state != lastState)
		{
			String state = "";
			if (m_state == 0)
			{
				state = "Idle";
			}//if
			if ((m_state & SpriteStates.SPRITESTATE_FALLING) != 0)
			{
				state += "Falling|";
			}//if
			if ((m_state & SpriteStates.SPRITESTATE_LEAPING) != 0)
			{
				state += "Leaping|";
			}//if
			if ((m_state & SpriteStates.SPRITESTATE_WALKING) != 0)
			{
				state += "Walking|";
			}//if
			if ((m_state & SpriteStates.SPRITESTATE_MOVING_LEFT) != 0)
			{
				state += "Left";
			}//if
			if ((m_state & SpriteStates.SPRITESTATE_MOVING_RIGHT) != 0)
			{
				state += "Right";
			}//if
			System.out.println(state);
			lastState = m_state;
		}//if*/
	}

	public void setDisplaySize(int w, int h)
	{
		m_displayWidth = w;
		m_displayHeight = h;
	}
}